diff --git a/common/common.c b/common/common.c
index 9260c64..1869034 100644
--- a/common/common.c
+++ b/common/common.c
@@ -313,7 +313,10 @@ int x264_param_parse( x264_param_t *p, const char *name, const char *value )
         }
     }
     OPT2("ref", "frameref")
-        p->i_frame_reference = atoi(value);
+        if( !strcmp(value, "auto") )
+            p->i_frame_reference = X264_FRAME_REFERENCE_AUTO;
+        else
+            p->i_frame_reference = atoi(value);
     OPT("keyint")
     {
         p->i_keyint_max = atoi(value);
diff --git a/encoder/encoder.c b/encoder/encoder.c
index fdcc957..2aae904 100644
--- a/encoder/encoder.c
+++ b/encoder/encoder.c
@@ -456,7 +456,15 @@ static int x264_validate_parameters( x264_t *h )
                   h->param.i_width, h->param.i_height );
     }
 
+    if( h->param.i_frame_reference == X264_FRAME_REFERENCE_AUTO )
+    {
+        const x264_level_t *l = x264_levels;
+        while( l[1].level_idc && l->level_idc != h->param.i_level_idc )
+            l++;
+        h->param.i_frame_reference = l->dpb / (((h->param.i_width+15)>>4) * ((h->param.i_height+15)>>4) * 384);
+    }
     h->param.i_frame_reference = x264_clip3( h->param.i_frame_reference, 1, 16 );
+
     if( h->param.i_keyint_max <= 0 )
         h->param.i_keyint_max = 1;
     if( h->param.i_scenecut_threshold < 0 )
diff --git a/x264.h b/x264.h
index e61040e..869dd5f 100644
--- a/x264.h
+++ b/x264.h
@@ -136,6 +136,9 @@ static const char * const x264_colmatrix_names[] = { "GBR", "bt709", "undef", ""
 /* Threading */
 #define X264_THREADS_AUTO 0 /* Automatically select optimal number of threads */
 
+/* Reference frames */
+#define X264_FRAME_REFERENCE_AUTO 0 /* Select the maximum number of reference frames allowed for the target level */
+
 /* Zones: override ratecontrol or other options for specific sections of the video.
  * See x264_encoder_reconfig() for which options can be changed.
  * If zones overlap, whichever comes later in the list takes precedence. */
